"""
synapse-diaspora-auth: A diaspora authenticator for matrix synapse.
Copyright (C) 2017-2018 Shamil K Muhammed.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from twisted.internet import defer, threads
import synapse

import bcrypt

import logging

from pkg_resources import parse_version


__VERSION__ = "0.2.2"

logger = logging.getLogger(__name__)


class DiasporaAuthProvider:
    __version__ = __VERSION__

    def __init__(self, config, account_handler):
        self.account_handler = account_handler
        self.config = config
        self.auth_handler = self.account_handler._auth_handler
        if self.config.engine == "mysql":
            import pymysql

            self.module = pymysql
        elif self.config.engine == "postgres":
            import psycopg2

            self.module = psycopg2

    @defer.inlineCallbacks
    def exec_query(self, query, *args):
        self.connection = self.module.connect(
            database=self.config.db_name,
            user=self.config.db_username,
            password=self.config.db_password,
            host=self.config.db_host,
            port=self.config.db_port,
        )
        try:
            with self.connection:
                with self.connection.cursor() as cursor:
                    yield threads.deferToThread(  # Don't think this is needed, but w/e
                        cursor.execute,
                        query,
                        args,
                    )
                    results = yield threads.deferToThread(cursor.fetchall)
                    cursor.close()
            defer.returnValue(results)
        except self.module.Error as e:
            logger.warning("Error during execution of query: {}: {}".format(query, e))
            defer.returnValue(None)
        finally:
            self.connection.close()

    @defer.inlineCallbacks
    def check_password(self, user_id, password):
        if not password:
            defer.returnValue(False)
        local_part = user_id.split(":", 1)[0][1:]
        users = yield self.exec_query(
            "SELECT username, encrypted_password, email FROM users WHERE username=%s",
            local_part,
        )
        # user_id is @localpart:hs_bare. we only need the localpart.
        logger.info("Checking if user {} exists.".format(local_part))

        # check if the user exists.
        if not users:
            logger.info(
                "User {} does not exist. Rejecting auth request".format(local_part)
            )
            defer.returnValue(False)
        user = users[0]
        logger.debug("User {} exists. Checking password".format(local_part))
        # user exists, check if the password is correct.
        encrypted_password = user[1]
        email = user[2]
        peppered_pass = "{}{}".format(password, self.config.pepper)
        if not (
            bcrypt.hashpw(
                peppered_pass.encode("utf8"), encrypted_password.encode("utf8")
            )
            == encrypted_password.encode("utf8")
        ):
            logger.info(
                "Password given for {} is wrong. Rejecting auth request.".format(
                    local_part
                )
            )
            defer.returnValue(False)
        logger.info("Confirming authentication request.")
        defer.returnValue(True)

    @staticmethod
    def parse_config(config):
        class _Conf:
            pass

        Conf = _Conf()
        Conf.engine = config["database"]["engine"]
        Conf.db_name = (
            "diaspora_production"
            if not config["database"]["name"]
            else config["database"]["name"]
        )
        Conf.db_host = config["database"]["host"]
        Conf.db_port = config["database"]["port"]
        Conf.db_username = config["database"]["username"]
        Conf.db_password = config["database"]["password"]
        Conf.pepper = config["pepper"]
        return Conf
